import pandas as pd
import geopandas as gpd

from sqlalchemy import text

class GeolevelDataType:
    # Constructor
    def __init__(
            self,
            custdata_type_id:int,
            name:str,
            properties:dict,
            geolevel_id:int,
            org:'Org' # type: ignore
    ):
        self.geolevel_data_type_id = custdata_type_id
        self.name = name
        self.properties = properties
        self.geolevel_id = geolevel_id
        self.org = org

    # Getters
    @property
    def geoleveldata_type_id(self): return self._geoleveldata_type_id
    @property
    def name(self): return self._name
    @property
    def properties(self): return self._properties
    @property
    def geolevel_id(self): return self._geolevel_id
    @property
    def org(self): return self._org



def about():
    print("This is a simple Python package to interract with Galigeo's products")
    print("Author: Jules Pierrat")
    print("License: Apache License 2.0")
    print("Code source: https://github.com/JPTGGO/galigeopy")
    print("Documentation: https://galigeopy.readthedocs.io/en/latest/")

def check():
    return True


import unittest
import json
from sqlalchemy import Engine

from galigeopy.org.org import Org
from galigeopy.model.network import Network
from galigeopy.model.zone_type import ZoneType
from galigeopy.model.zone import Zone

class TestOrg(unittest.TestCase):

    def __init__(self, methodName: str = "runTest") -> None:
        super().__init__(methodName)
        self.conf = json.load(open("test-config.json"))

    def test_org(self):
        # Valid Org
        org = Org(**self.conf["org"])
        self.assertTrue(org.is_valid)
        self.assertIsNotNone(org.engine)
        self.assertIsInstance(org.engine, Engine)
        del org
        # Invalid
        org = Org(user="xxx", password="xxx")
        self.assertFalse(org.is_valid)
        del org

    def test_query(self):
        # Valid Org
        org = Org(**self.conf["org"])
        # Query
        df = org.query("SELECT * FROM ggo_network")
        self.assertIsNotNone(df)
        self.assertGreater(len(df), 0)

    def test_get_networks_list(self):
        # Valid Org
        org = Org(**self.conf["org"])
        # Get networks list
        df = org.getNetworksList()
        self.assertIsNotNone(df)
        self.assertGreater(len(df), 0)
        del org

    def test_get_network_by_id(self):
        # Valid Org
        org = Org(**self.conf["org"])
        # Get network by id
        network = org.getNetworkById(self.conf["network_id"])
        self.assertIsNotNone(network)
        self.assertIsInstance(network, Network)
        self.assertEqual(network.network_id, self.conf["network_id"])
        del org

    def test_get_network_by_name(self):
        # Valid Org
        org = Org(**self.conf["org"])
        name = org.getNetworkById(self.conf["network_id"]).name
        # Get network by name
        network = org.getNetworkByName(name)
        self.assertIsNotNone(network)
        self.assertIsInstance(network, Network)
        self.assertEqual(network.name, name)
        self.assertEqual(network.network_id, self.conf["network_id"])
        del org

    def test_get_all_networks(self):
        # Valid Org
        org = Org(**self.conf["org"])
        # Get all networks
        networks = org.getAllNetworks()
        self.assertIsNotNone(networks)
        self.assertGreater(len(networks), 0)
        self.assertIsInstance(networks[0], Network)
        del org

    def test_get_zone_types_list(self):
        # Valid Org
        org = Org(**self.conf["org"])
        # Get zone types list
        df = org.getZoneTypesList()
        self.assertIsNotNone(df)
        self.assertGreater(len(df), 0)
        del org

    def test_get_zone_type_by_id(self):
        # Valid Org
        org = Org(**self.conf["org"])
        zone_type = org.getZoneTypeById(self.conf["zone_type_id"])
        self.assertIsNotNone(zone_type)
        self.assertEqual(zone_type.zone_type_id, self.conf["zone_type_id"])
        del org

    def test_get_zone_type_by_name(self):
        # Valid Org
        org = Org(**self.conf["org"])
        name = org.getZoneTypeById(self.conf["zone_type_id"]).name
        # Get zone type by name
        zone_type = org.getZoneTypeByName(name)
        self.assertIsNotNone(zone_type)
        self.assertEqual(zone_type.name, name)
        self.assertEqual(zone_type.zone_type_id, self.conf["zone_type_id"])
        del org

    def test_get_all_zone_types(self):
        # Valid Org
        org = Org(**self.conf["org"])
        # Get all zone types
        zone_types = org.getAllZoneTypes()
        self.assertIsNotNone(zone_types)
        self.assertGreater(len(zone_types), 0)
        self.assertIsInstance(zone_types[0], ZoneType)
        del org

    def test_get_geolevels_list(self):
        # Valid Org
        org = Org(**self.conf["org"])
        # Get geolevels list
        df = org.getGeolevelsList()
        self.assertIsNotNone(df)
        self.assertGreater(len(df), 0)
        del org


import unittest
import json

from galigeopy.org.org import Org
from galigeopy.model.poi import Poi

class TestNetwork(unittest.TestCase):

    def __init__(self, methodName: str = "runTest") -> None:
        super().__init__(methodName)
        self.conf = json.load(open("test-config.json"))
        self.org = Org(**self.conf["org"])

    def test_network(self):
        network = self.org.getNetworkById(self.conf["network_id"])
        self.assertIsNotNone(network)
        self.assertEqual(network.org.engine, self.org.engine)

    def test_number_of_pois(self):
        network = self.org.getNetworkById(self.conf["network_id"])
        self.assertGreater(network.number_of_pois(), 0)

    def test_get_pois_list(self):
        network = self.org.getNetworkById(self.conf["network_id"])
        df = network.getPoisList()
        self.assertIsNotNone(df)
        self.assertGreater(len(df), 0)

    def test_get_poi_by_code(self):
        network = self.org.getNetworkById(self.conf["network_id"])
        poi = network.getPoiByCode("0020500041")
        self.assertIsNotNone(poi)
        self.assertEqual(poi.id, "0020500041")
        self.assertIsNotNone(poi.x)
        self.assertIsNotNone(poi.y)
        self.assertEqual(poi.network_id, network.network_id)

    def test_get_all_pois(self):
        network = self.org.getNetworkById(self.conf["network_id"])
        pois = network.getAllPois()
        self.assertIsNotNone(pois)
        self.assertGreater(len(pois), 0)
        self.assertIsInstance(pois[0], Poi)
        self.assertEqual(pois[0].network_id, network.network_id)




import pandas as pd
from sqlalchemy import text

from galigeopy.model.poi import Poi
from galigeopy.model.zone_geounit import ZoneGeounit

class Zone:
    # Constructor
    def __init__(
        self,
        zone_id:int,
        properties:dict,
        geolevel_id:int,
        zone_type_id:int,
        poi_id:int,
        parent_zone_id:int,
        org:'Org' # type: ignore
    ):
        # Infos
        self._zone_id = zone_id
        self._properties = properties
        self._geolevel_id = geolevel_id
        self._zone_type_id = zone_type_id
        self._poi_id = poi_id
        self._parent_zone_id = parent_zone_id
        # Org
        self._org = org

    # Getters and setters
    @property
    def zone_id(self): return self._zone_id
    @property
    def properties(self): return self._properties
    @property
    def geolevel_id(self): return self._geolevel_id
    @property
    def zone_type_id(self): return self._zone_type_id
    @property
    def poi_id(self): return self._poi_id
    @property
    def parent_zone_id(self): return self._parent_zone_id
    @property
    def org(self): return self._org

    # Public Methods
    def getPoi(self)->Poi:
        query = f"SELECT * FROM ggo_poi WHERE poi_id = {self.poi_id}"
        df = pd.read_sql(query, self._org.engine)
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self._org})
            return Poi(**data)
        else:
            raise Warning(f"Poi {self.poi_id} not found in Zone {self.zone_id}")
        
    def getParentZone(self) -> 'Zone':
        query = f"SELECT * FROM ggo_zone WHERE zone_id = {self.parent_zone_id}"
        df = pd.read_sql(query, self._org.engine)
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self._org})
            return Zone(**data)
        else:
            raise Warning(f"Parent Zone {self.parent_zone_id} not found in Zone {self.zone_id}")
        
    def getChildrenZones(self)->list:
        query = f"SELECT * FROM ggo_zone WHERE parent_zone_id = {self.zone_id}"
        df = pd.read_sql(query, self._org.engine)
        zones = []
        for i in range(len(df)):
            data = df.iloc[i].to_dict()
            data.update({"org": self._org})
            zones.append(Zone(**data))
        return zones
    
    def getZoneGeounitsList(self) -> pd.DataFrame:
        query = f"SELECT * FROM ggo_zone_geounit WHERE zone_id = {self.zone_id}"
        return pd.read_sql(query, self._org.engine)
    
    def getZoneGeounitById(self, geounit_id:int) -> pd.DataFrame:
        query = f"SELECT * FROM ggo_zone_geounit WHERE zone_id = {self.zone_id} AND zone_geounit_id = {geounit_id}"
        df = pd.read_sql(query, self._org.engine)
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self._org})
            return ZoneGeounit(**data)
        else:
            raise Warning(f"Geounit {geounit_id} not found in Zone {self.zone_id}")

    def add_to_model(self) -> int:
        # Add to database
        query = f"""
        INSERT INTO ggo_zone (
            properties,
            geolevel_id,
            zone_type_id,
            poi_id,
            parent_zone_id
        ) VALUES (
            '{self.properties}',
            {self.geolevel_id},
            {self.zone_type_id},
            {self.poi_id},
            {self.parent_zone_id if self.parent_zone_id is not None else 'NULL'}
        ) RETURNING zone_id;
        """
        zone_id = self._org.query(query)[0][0]
        return zone_id


# GaligeoPy
from galigeopy.galigeopy import about, check

# Org
from galigeopy.org.org import Org

# Model
from galigeopy.model.network import Network
from galigeopy.model.poi import Poi
from galigeopy.model.zone_type import ZoneType
from galigeopy.model.zone import Zone
from galigeopy.model.zone_geounit import ZoneGeounit
from galigeopy.model.geolevel import Geolevel


# IMPORTS
from sqlalchemy import create_engine, Engine, text
import pandas as pd

from galigeopy.model.network import Network
from galigeopy.model.poi import Poi
from galigeopy.model.zone_type import ZoneType
from galigeopy.model.geolevel import Geolevel
from galigeopy.model.geolevel_data_type import GeolevelDataType

class Org:
    # Constructor
    def __init__(self, user, password, host="127.0.0.1", port=5432, db="galigeo"):
        # Infos
        self._user = user
        self._password = password
        self._host = host
        self._port = port
        self._db = db
        # Engine
        self._is_valid = False
        self._engine = self._get_engine()
        self._is_valid = self._check_validity()
        self._default_schema = self._get_default_schema()
    
    # Getters and setters
    @property
    def engine(self): return self._engine
    @property
    def is_valid(self): return self._is_valid
    @property
    def default_schema(self): return self._default_schema

    # Private Methods
    def _get_engine(self) -> Engine:
        return create_engine(f'postgresql://{self._user}:{self._password}@{self._host}:{self._port}/{self._db}')

    def _get_default_schema(self)->str:
        query = "SELECT current_schema()"
        return self.query(query)[0][0]

    def _check_validity(self)->bool:
        # Check connection
        try:
            self._engine.connect()
            return True
        except Exception as e:
            return False
    
    # Public Methods
    def query_df(self, query:str)->pd.DataFrame:
        return pd.read_sql(query, self._engine)

    def query(self, query:str)->list:
        with self._engine.connect() as conn:
            result = conn.execute(text(query))
            conn.commit()
            if result.returns_rows:
                return result.fetchall()
            else:
                return []

    def getNetworksList(self)->pd.DataFrame:
        # Query
        query = "SELECT * FROM ggo_network"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # return df
        return df
    
    def getNetworkById(self, id:int)->Network:
        # Query
        query = f"SELECT * FROM ggo_network WHERE network_id = {str(id)}"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return Network(**data)
        else:
            raise Warning(f"Network with id {id} not found")
    
    def getNetworkByName(self, name:str)->Network:
        # Query
        query = f"SELECT * FROM ggo_network WHERE name = '{name}'"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return Network(**data)
        else:
            raise Warning(f"Network with name {name} not found")
    
    def getAllNetworks(self)->list:
        # Query
        query = "SELECT * FROM ggo_network"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        networks = []
        for i in range(len(df)):
            data = df.iloc[i].to_dict()
            data.update({"org": self})
            networks.append(Network(**data))
        return networks
    
    def getPoisByCode(self, code:str)->list:
        # Query
        query = f"SELECT * FROM ggo_poi WHERE id = '{code}'"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        pois = []
        for i in range(len(df)):
            data = df.iloc[i].to_dict()
            data.update({"org": self})
            pois.append(Poi(**data))
        return pois
    
    def getZoneTypesList(self)->pd.DataFrame:
        # Query
        query = "SELECT * FROM ggo_zone_type"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # return df
        return df
    
    def getZoneTypeById(self, id:int)->ZoneType:
        # Query
        query = f"SELECT * FROM ggo_zone_type WHERE zone_type_id = {str(id)}"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return ZoneType(**data)
        else:
            raise Warning(f"ZoneType with id {id} not found")
        
    def getZoneTypeByName(self, name:str)->ZoneType:
        # Query
        query = f"SELECT * FROM ggo_zone_type WHERE name = '{name}'"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return ZoneType(**data)
        else:
            raise Warning(f"ZoneType with name {name} not found")
        
    def getAllZoneTypes(self)->list:
        # Query
        query = "SELECT * FROM ggo_zone_type"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        zone_types = []
        for i in range(len(df)):
            data = df.iloc[i].to_dict()
            data.update({"org": self})
            zone_types.append(ZoneType(**data))
        return zone_types
    
    def getGeolevelsList(self)->pd.DataFrame:
        # Query
        query = "SELECT * FROM ggo_geolevel"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # return df
        return df
    
    def getGeolevelById(self, id:int)->Geolevel:
        # Query
        query = f"SELECT * FROM ggo_geolevel WHERE geolevel_id = {str(id)}"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return Geolevel(**data)
        else:
            raise Warning(f"Geolevel with id {id} not found")

    def getGeolevelByName(self, name:str)->Geolevel:
        # Query
        query = f"SELECT * FROM ggo_geolevel WHERE name = '{name}'"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return Geolevel(**data)
        else:
            raise Warning(f"Geolevel with name {name} not found")
        
    def getAllGeolevels(self)->list:
        # Query
        query = "SELECT * FROM ggo_geolevel"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        geolevels = []
        for i in range(len(df)):
            data = df.iloc[i].to_dict()
            data.update({"org": self})
            geolevels.append(Geolevel(**data))
        return geolevels
    
    def getGeolevelDataTypesList(self)->pd.DataFrame:
        # Query
        query = "SELECT * FROM ggo_geoleveldata_type"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # return df
        return df
    
    def getGeolevelDataTypeById(self, id:int)->GeolevelDataType:
        # Query
        query = f"SELECT * FROM ggo_geoleveldata_type WHERE custdata_type_id = {str(id)}"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return GeolevelDataType(**data)
        else:
            raise Warning(f"GeolevelDataType with id {id} not found")
        
    def getGeolevelDataTypeByName(self, name:str)->GeolevelDataType:
        # Query
        query = f"SELECT * FROM ggo_geoleveldata_type WHERE name = '{name}'"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return GeolevelDataType(**data)
        else:
            raise Warning(f"GeolevelDataType with name {name} not found")